﻿using System;
using System.Collections.Generic;

namespace CollectionSample
{
    class Program
    {
        static void Main(String[] args)
        {
            List<Int32> list = new List<Int32>(10) { 5, 2, 1, 3, 4 };
            Console.WriteLine("Length: {0}, Capacity: {1}", list.Count, list.Capacity);

            Console.Write("Elements:");
            for (Int32 i = 0; i < list.Count; ++i)
                Console.Write(" " + list[i]);
            Console.WriteLine();

            list.Add(7);
            list.AddRange(new Int32[] { 9, 5, 10, 4, 7, 3, 8 });
            list.Insert(3, 8);

            Console.Write("Elements:");
            foreach (Int32 item in list)
                Console.Write(" " + item);
            Console.WriteLine();

            Console.WriteLine("Contains:");
            for (Int32 i = 0; i <= 10; ++i)
                Console.WriteLine("\tList contains {0}: {1}", i, list.Contains(i) ? "yes" : "no");

            Int32 result = list.Find(item => item > 5);
            Console.WriteLine("First element greater than 5 is {0}", result);

            list.Sort(new MyComparer());

            Console.Write("Elements:");
            list.ForEach(item => Console.Write(" " + item));
            Console.WriteLine();

            list.RemoveAll(item => item % 3 == 0);
            list.Reverse();

            IList<Int32> ilist = list;
            ICollection<Int32> icollection = ilist;
            IEnumerable<Int32> ienumerable = icollection;
            Console.Write("Elements:");
            foreach (Int32 item in ienumerable)
                Console.Write(" " + item);
            Console.WriteLine();

            LinkedList<Int32> linkedList = new LinkedList<Int32>(list);
            linkedList.AddFirst(3);
            Console.WriteLine("The last but one element: " + linkedList.Last.Previous.Value);
            icollection = linkedList;

            // ---------------------------
            Console.ReadKey();
            Console.WriteLine();
            // ---------------------------

            Stack<String> stack = new Stack<String>();
            stack.Push("apple");
            stack.Push("orange");
            stack.Push("banana");
            stack.Push("pear");
            stack.Push("lemon");

            Console.Write("Elements:");
            foreach (String item in stack)
                Console.Write(" " + item);
            Console.WriteLine();

            Queue<String> queue = new Queue<String>(stack);
            queue.Enqueue("peach");
            queue.Enqueue("pineapple");

            Console.Write("Elements:");
            while (queue.Count > 0)
            {
                Console.Write(" " + queue.Peek());
                queue.Dequeue();
            }
            Console.WriteLine();

            // ---------------------------
            Console.ReadKey();
            Console.WriteLine();
            // ---------------------------

            Dictionary<String, Person> dictionary = new Dictionary<String, Person>();
            dictionary.Add("jani", new Person() { Name = "Kovács János", Age = 26 });
            dictionary.Add("gabi", new Person() { Name = "Tímár Gabriella", Age = 19 });
            dictionary.Add("feri", new Person() { Name = "Kiss Ferenc", Age = 32 });
            dictionary.Add("pityu", new Person() { Name = "Nagy István", Age = 63 });

            try
            {
                dictionary.Add("feri", new Person() { Name = "Gipsz Ferenc", Age = 41 });
            }
            catch (ArgumentException)
            {
                Console.WriteLine("The key 'feri' already exists.");
            }

            dictionary["feri"] = new Person() { Name = "Gipsz Ferenc", Age = 41 };
            dictionary["feri"].Age = 47;
            dictionary["juli"] = new Person() { Name = "Varga Júlia", Age = 23 };

            try
            {
                dictionary["agi"].Name = "Németh Ágnes";
            }
            catch (KeyNotFoundException)
            {
                Console.WriteLine("The key 'agi' does not exist.");
            }

            if (dictionary.ContainsKey("jani"))
                Console.WriteLine("The key 'jani' exists.");

            if (dictionary.ContainsValue(new Person() { Name = "Nagy István", Age = 63 }))
                Console.WriteLine("The requested new object exists  in the dictionary.");
            else
                Console.WriteLine("The requested new object doesn not exist in the dictionary.");

            Person pityu = dictionary["pityu"];
            if (dictionary.ContainsValue(pityu))
                Console.WriteLine("The requested referenced object exists in the dictionary.");
            else
                Console.WriteLine("The requested referenced object does not exist in the dictionary.");

            dictionary.Remove("gabi");

            Console.Write("Keys:");
            foreach (String key in dictionary.Keys)
                Console.Write(" " + key);
            Console.WriteLine();

            Console.Write("Elements:");
            foreach (KeyValuePair<String, Person> item in dictionary)
                Console.Write(" " + item);
            Console.WriteLine();

            // ---------------------------
            Console.ReadKey();
            Console.WriteLine();
            // ---------------------------

            // HashSet<T>
            // SortedList<K, V>
            // SortedDictionary<K, V>
            // SortedSet<T>             
        }
    }
}
 