﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace CustomEnumerable
{
    class RandomEnumerable<T> : IEnumerable<T>
    {
        List<T> elements;

        public RandomEnumerable()
        {
            elements = new List<T>();
        }

        public void Add(T item)
        {
            elements.Add(item);
        }

        public Boolean Remove(T item)
        {
            return elements.Remove(item);
        }

        #region IEnumerable<T> interface

        public IEnumerator<T> GetEnumerator()
        {
            elements.Sort(new RandomComparer());
            foreach (T item in elements)
                yield return item;
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion

        private class RandomComparer : IComparer<T>
        {
            private static Random generator = new Random();

            public Int32 Compare(T x, T y)
            {
                return generator.Next(-1, 2);
            }
        }
    }
}
